-------------------------------------------------
-- Resource Tooltip Generator
-------------------------------------------------

function GenerateResourceToolTip(plot)
	local bShowBasicHelp = not OptionsManager.IsNoBasicHelp()

	-- Resource
    local iResourceID = plot:GetResourceType(Game.GetActiveTeam());
    local pResourceInfo = GameInfo.Resources[iResourceID];
    if (pResourceInfo == nil) then
		return nil;
	end
    
    -- Quantity
    local strQuantity = "";
    if (plot:GetNumResource() > 1) then
		strQuantity = plot:GetNumResource() .. " ";
    end

    
    -- Name
    local strToolTip = "[COLOR_POSITIVE_TEXT]" .. strQuantity .. Locale.ToUpper(Locale.ConvertTextKey(pResourceInfo.Description)) .. "[ENDCOLOR]";

	if bShowBasicHelp then
		-- Extra Help Text (e.g. for strategic resources)
		if pResourceInfo.Help then
			-- Basic tooltips get extra info
			if bShowBasicHelp then
				strToolTip = strToolTip .. "[NEWLINE]";
				strToolTip = strToolTip .. "[ICON_BULLET]" .. Locale.ConvertTextKey(pResourceInfo.Help);
				strToolTip = strToolTip .. "[NEWLINE]---------------------";
			end
		end
	
		-- Happiness
		if (pResourceInfo.Happiness) then
			if (pResourceInfo.Happiness ~= 0) then
				local strHappinessToolTip = "";

				-- Basic tooltips get extra info
				if bShowBasicHelp then
					strHappinessToolTip = Locale.ConvertTextKey("TXT_KEY_RESOURCE_TOOLTIP_IMPROVED");
					strHappinessToolTip = strHappinessToolTip .. "[NEWLINE]";
				end
			
				strHappinessToolTip = strHappinessToolTip .. "[ICON_BULLET]+" .. pResourceInfo.Happiness .. " [ICON_HAPPINESS_1]";
			
				-- Basic tooltips get extra info
				if bShowBasicHelp then
					strHappinessToolTip = strHappinessToolTip .. " " .. Locale.ConvertTextKey("TXT_KEY_GAME_CONCEPT_SECTION_10");	-- This is the text key for Happiness... don't ask
				end

				strToolTip = strToolTip .. "[NEWLINE]";
				strToolTip = strToolTip .. strHappinessToolTip;
			end
		end
	
		-- Yield
		local strYieldToolTip = "";
		local condition = "ResourceType = '" .. pResourceInfo.Type .. "'";
		local pYieldInfo;
		local bFirst = true;
		for row in GameInfo.Resource_YieldChanges( condition ) do
			pYieldInfo = GameInfo.Yields[row.YieldType];
		
			-- Add bullet in front of first entry, space in front of all others
			if (bFirst) then
				bFirst = false;
				strYieldToolTip = strYieldToolTip .. "[ICON_BULLET]";
			else
				strYieldToolTip = strYieldToolTip .. " ";
			end
		
			if row.Yield > 0 then
				strYieldToolTip = strYieldToolTip.."+";
			end
		
			strYieldToolTip = strYieldToolTip .. tostring(row.Yield) .. pYieldInfo.IconString;
		
			-- Basic tooltips get extra info
			if bShowBasicHelp then
				strYieldToolTip = strYieldToolTip .. " " .. Locale.ConvertTextKey(pYieldInfo.Description);
			end
		end

		-- Something in the yield tooltip?
		if (strYieldToolTip ~= "") then
		
			-- Basic tooltips get extra info
			if bShowBasicHelp then
				strYieldToolTip = Locale.ConvertTextKey("TXT_KEY_RESOURCE_TOOLTIP_IMPROVED_WORKED") .. "[NEWLINE]" .. strYieldToolTip;
			end
		
			strToolTip = strToolTip .. "[NEWLINE]";
			strToolTip = strToolTip .. strYieldToolTip;
		end
	end
	
	return strToolTip;
end
